@@ -1,24 +1,34 @@
-/* This is a Glade front end to the little phase field/fluid structure
-   interactions program based on PETSc, but can serve as a front end to a
-   variety of PETSc programs with minor adjustments.
+/***************************************
+  $Header: /cvsroot/petscgraphics/chui.c,v 1.13 2004/05/25 13:27:27 hazelsct Exp $
 
-   Callback functions are grouped here according to where they appear in the
-   main window (except on_run_clicked is below main window items), then the run
-   control dialog, and save dialog. */
+  This is a Glade front end to the little phase field/fluid structure
+  interactions program based on PETSc, but can serve as a front end to a
+  variety of PETSc programs with minor adjustments.
 
-#include <gtk/gtk.h>
+  Callback functions are grouped here according to where they appear in the
+  main window (except on_run_clicked is below main window items), then the run
+  control dialog, and save dialog.
+
+  I haven't put function comments in to keep the line count down, the functions
+  are generally pretty simple GTK+ callbacks.
+***************************************/
+
+
+/* Includes, using GTK_ENABLE_BROKEN to enable the GtkText widget for now. */
 #include <glade/glade.h>
+#include <gnome.h>
+#include <libgnomeui/libgnomeui.h>
 #include <stdio.h>
 #include <math.h>
 
 GladeXML *xml;
 FILE *simulation_input_file, *simulation_output_file;
 
-/* Simple debugging macro, set to one for debugging output and zero for none */
-#if 0
+/* Simple debugging macro */
+#ifdef DEBUG
 #define DPRINTF printf
 #else
-#define DPRINTF(...)
+#define DPRINTF(fmt...)
 #endif
 
 /* Discretization stuff */
@@ -26,17 +36,14 @@ double Lx;
 int nx;
 
 void on_width_changed (GtkWidget *width, gpointer user_data) {
-  gchar *entrytext;
+  G_CONST_RETURN gchar *entrytext;
   entrytext = gtk_entry_get_text (GTK_ENTRY (width));
   sscanf (entrytext, "%lf", &Lx);
   DPRINTF ("Width set to %g\n", Lx); }
 
 void on_resolution_changed (GtkWidget *resolution, gpointer user_data) {
   /* Note: gtk_spin_button_get_value_as_int doesn't respond to typed entry */
-  /* nx = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (resolution)); */
-  gchar *entrytext;
-  entrytext = gtk_entry_get_text (GTK_ENTRY (resolution));
-  sscanf (entrytext, "%d", &nx);
+  nx = gtk_spin_button_get_value (GTK_SPIN_BUTTON (resolution));
   DPRINTF ("Resolution set to %d\n", nx); }
 
 /* Timestep stuff */
@@ -44,27 +51,26 @@ double dt, dt_factor, dt_max;
 int last_tstep;
 
 void on_timestep_changed (GtkWidget *timestep, gpointer user_data) {
-  gchar *entrytext;
+  G_CONST_RETURN gchar *entrytext;
   entrytext = gtk_entry_get_text (GTK_ENTRY (timestep));
   sscanf (entrytext, "%lf", &dt);
   DPRINTF ("Timestep set to %g\n", dt); }
 
 void on_time_factor_changed (GtkWidget *time_factor, gpointer user_data) {
-  gchar *entrytext;
+  G_CONST_RETURN gchar *entrytext;
   entrytext = gtk_entry_get_text (GTK_ENTRY (time_factor));
   sscanf (entrytext, "%lf", &dt_factor);
   DPRINTF ("Time factor set to %g\n", dt_factor); }
 
 void on_max_timestep_changed (GtkWidget *max_timestep, gpointer user_data) {
-  gchar *entrytext;
+  G_CONST_RETURN gchar *entrytext;
   entrytext = gtk_entry_get_text (GTK_ENTRY (max_timestep));
   sscanf (entrytext, "%lf", &dt_max);
   DPRINTF ("Max timestep set to %g\n", dt_max); }
 
 void on_last_timestep_changed (GtkWidget *last_timestep, gpointer user_data) {
-  gchar *entrytext;
-  entrytext = gtk_entry_get_text (GTK_ENTRY (last_timestep));
-  sscanf (entrytext, "%d", &last_tstep);
+  /* Note: gtk_spin_button_get_value_as_int doesn't respond to typed entry */
+  last_tstep = gtk_spin_button_get_value (GTK_SPIN_BUTTON (last_timestep));
   DPRINTF ("Last timestep set to %d\n", last_tstep); }
 
 /* Display stuff */
@@ -80,7 +86,8 @@ void on_textdisplay_toggled (GtkWidget *textdisplay, gpointer user_data) {
 
 /* Remote host stuff */
 gboolean remote_host;
-gchar *remote_hostname, thetransport[50];
+G_CONST_RETURN gchar *remote_hostname;
+gchar thetransport[50];
 
 void on_remote_check_toggled (GtkWidget *remote_check, gpointer user_data) {
   if (remote_host = gtk_toggle_button_get_active
@@ -117,7 +124,7 @@ void on_remote_host_changed (GtkWidget *remote_host, gpointer user_data) {
   DPRINTF ("Remote host set to %s\n", remote_hostname); }
 
 /* MPI stuff */
-gchar *mpirun_command;
+G_CONST_RETURN gchar *mpirun_command;
 int number_cpus;
 
 void on_mpirun_changed (GtkWidget *mpirun, gpointer user_data) {
@@ -126,10 +133,7 @@ void on_mpirun_changed (GtkWidget *mpirun, gpointer user_data) {
 
 void on_num_cpus_changed (GtkWidget *num_cpus, gpointer user_data) {
   /* Note: gtk_spin_button_get_value_as_int doesn't respond to typed entry */
-  /* nx = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (num_cpus)); */
-  gchar *entrytext;
-  entrytext = gtk_entry_get_text (GTK_ENTRY (num_cpus));
-  sscanf (entrytext, "%d", &number_cpus);
+  number_cpus = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (num_cpus));
   if (number_cpus > 1) {
     gtk_widget_set_sensitive
       (glade_xml_get_widget (xml, "mpirun_label"), TRUE);
@@ -152,6 +156,7 @@ void on_3d_activate (GtkWidget *unused, gpointer user_data) {
   twodee = FALSE; DPRINTF ("Simulating in three dimensions\n"); }
 
 /* Load and save (unfinished!) */
+gchar *options_filename = NULL;
 void on_load_ok_clicked (GtkWidget *load_file, gpointer user_data) {
   gtk_widget_hide (load_file); }
 
@@ -159,20 +164,97 @@ void on_save_ok_clicked (GtkWidget *save_file, gpointer user_data) {
   gtk_widget_hide (save_file); }
 
 /* Toggle buttons */
-void on_show_options_toggled (GtkWidget *options_notebook, gpointer user_data){
-  if (gtk_toggle_button_get_active
-      (GTK_TOGGLE_BUTTON (glade_xml_get_widget (xml, "show_options"))))
-    gtk_widget_show (options_notebook);
-  else
-    gtk_widget_hide (options_notebook); }
+void on_show_options_toggled (GtkWidget *show_options, gpointer user_data) {
+  GtkWidget *simulation_options =
+    glade_xml_get_widget (xml,"simulation_options");
+  GtkWidget *options_notebook = glade_xml_get_widget (xml,"options_notebook");
+  gboolean show_options_active =
+    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (show_options));
+  gboolean simulation_options_active =
+    gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (simulation_options));
+  DPRINTF ("on_show_options_toggled: show %d, simulation %d\n",
+	   show_options_active, simulation_options_active);
 
-void on_show_output_toggled (GtkWidget *output_window, gpointer user_data) {
-  if (gtk_toggle_button_get_active
-      (GTK_TOGGLE_BUTTON (glade_xml_get_widget (xml, "show_output"))))
+  if (show_options_active != simulation_options_active)
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (simulation_options),
+				  show_options_active);
+  if (show_options_active)
+    gtk_widget_show (options_notebook);
+  else {
+    GtkWidget *main_window = glade_xml_get_widget (xml, "main_window");
+    gtk_widget_hide (options_notebook);
+    gtk_window_resize (GTK_WINDOW (main_window), 1, 1); }
+}
+
+void on_show_output_toggled (GtkWidget *show_output, gpointer user_data) {
+  GtkWidget *simulation_output =
+    glade_xml_get_widget (xml,"simulation_output");
+  GtkWidget *output_window = glade_xml_get_widget (xml,"output_window");
+  gboolean show_output_active =
+    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (show_output));
+  gboolean simulation_output_active =
+    gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (simulation_output));
+  DPRINTF ("on_show_output_toggled: show %d, simulation %d\n",
+	   show_output_active, simulation_output_active);
+
+  if (show_output_active != simulation_output_active)
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (simulation_output),
+				  show_output_active);
+  if (show_output_active)
     gtk_widget_show (output_window);
   else
     gtk_widget_hide (output_window); }
 
+/* Menu items, mainly calling things above (SO annoying that menu signals don't
+   have associated objects!) */
+void open_params (GtkWidget *null_widget, gpointer user_data) {
+  GtkWidget *load_file = glade_xml_get_widget (xml,"load_file");
+  gtk_widget_show (load_file); }
+
+void save_params_as (GtkWidget *null_widget, gpointer user_data) {
+  GtkWidget *save_file = glade_xml_get_widget (xml,"save_file");
+  gtk_widget_show (save_file); }
+
+void save_params (GtkWidget *null_widget, gpointer user_data) {
+  if (options_filename == NULL)
+    save_params_as (null_widget, user_data); }
+
+void on_show_options_activate (GtkWidget *null_widget, gpointer user_data) {
+  GtkWidget *simulation_options =
+    glade_xml_get_widget (xml,"simulation_options");
+  GtkWidget *show_options = glade_xml_get_widget (xml,"show_options");
+  gboolean simulation_options_active =
+    gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (simulation_options));
+  gboolean show_options_active =
+    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (show_options));
+  DPRINTF ("on_show_options_activate: show %d, simulation %d\n",
+	   show_options_active, simulation_options_active);
+
+  /* They will be unequal if this one was activated */
+  if (simulation_options_active != show_options_active)
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (show_options),
+				  simulation_options_active); }
+
+void on_show_output_activate (GtkWidget *null_widget, gpointer user_data) {
+  GtkWidget *simulation_output =
+    glade_xml_get_widget (xml,"simulation_output");
+  GtkWidget *show_output = glade_xml_get_widget (xml,"show_output");
+  gboolean simulation_output_active =
+    gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (simulation_output));
+  gboolean show_output_active =
+    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (show_output));
+  DPRINTF ("on_show_output_activate: show %d, simulation %d\n",
+	   show_output_active, simulation_output_active);
+
+  /* They will be unequal if this one was activated */
+  if (simulation_output_active != show_output_active)
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (show_output),
+				  simulation_output_active); }
+
+void on_about_activate (GtkWidget *null_widget, gpointer user_data) {
+  GtkWidget *about_window = glade_xml_get_widget (xml,"about_window");
+  gtk_widget_show (about_window); }
+
 /* The run button! */
 gint pipe_input_tag; /* Tag number for gdk_input_add/remove */
 
@@ -279,20 +361,27 @@ void read_simulation_data (gpointer user_data, gint source,
     /* End of simulation */
     else if (!(strncmp (simulation_input, "Game over", 9)))
       on_stop_clicked
-	(glade_xml_get_widget (xml, "run_control_frame"), NULL);
+	(glade_xml_get_widget (xml, "output_window"), NULL);
 
     /* Write the text to the output widget in the run dialog */
     else
-      gtk_text_insert (GTK_TEXT (output_text), NULL, NULL, NULL,
-		       simulation_input, strlen (simulation_input)); }
+      {
+	GtkTextBuffer *buffer = gtk_text_view_get_buffer
+	  (GTK_TEXT_VIEW (output_text));
+	GtkTextIter iter;
+	gtk_text_buffer_get_end_iter (buffer, &iter);
+	gtk_text_buffer_insert
+	  (buffer, &iter, simulation_input, strlen (simulation_input));
+      }
+  }
 }
   
 
-#define MAX_COMMAND_LINE_OPTIONS 20
+#define MAX_COMMAND_LINE_OPTIONS 22
 
 int from_simulation_pipe[2], to_simulation_pipe[2];
 
-void on_run_clicked (GtkWidget *run_control_frame, gpointer user_data) {
+void on_run_activate (GtkWidget *null_widget, gpointer user_data) {
   gchar command_line [MAX_COMMAND_LINE_OPTIONS][50],
     clinestr [MAX_COMMAND_LINE_OPTIONS*50];
   int simulation_pid;
@@ -315,6 +404,8 @@ void on_run_clicked (GtkWidget *run_control_frame, gpointer user_data) {
      macros above, and also the execlp command below. */
   sprintf (command_line[command], CHTS_PATH);
   strcat (command_line[command++], "/chts");
+  sprintf (command_line[command++], "-ts_max_steps");
+  sprintf (command_line[command++], "%d", last_tstep);
   sprintf (command_line[command++], "-mx");
   sprintf (command_line[command++], "%d", nx);
   sprintf (command_line[command++], "-my");
@@ -340,8 +431,14 @@ void on_run_clicked (GtkWidget *run_control_frame, gpointer user_data) {
     sprintf (clinestr+strlen(clinestr), " %s", command_line [command]);
   sprintf (clinestr+strlen(clinestr), "\n");
   DPRINTF ("%s", clinestr);
-  gtk_text_insert (GTK_TEXT (glade_xml_get_widget (xml, "output_text")), NULL,
-		   NULL, NULL, clinestr, strlen (clinestr));
+  {
+    GtkTextBuffer *buffer = gtk_text_view_get_buffer
+      (GTK_TEXT_VIEW (glade_xml_get_widget (xml, "output_text")));
+    GtkTextIter iter;
+    gtk_text_buffer_get_end_iter (buffer, &iter);
+    gtk_text_buffer_insert
+      (buffer, &iter, clinestr, strlen (clinestr));
+  }
 
   /* Try to spawn a new simulation process.  Most of this was shamelessly
      ripped from Ken Brakke's Surface Evolver. */
@@ -370,7 +467,7 @@ void on_run_clicked (GtkWidget *run_control_frame, gpointer user_data) {
 	   command_line[7], command_line[8], command_line[9], command_line[10],
 	   command_line[11],command_line[12],command_line[13],command_line[14],
 	   command_line[15],command_line[16],command_line[17],command_line[18],
-	   command_line[19],NULL);
+	   command_line[19],command_line[20],command_line[21],NULL);
     perror (command_line[0]); /* only error gets here */
     exit (1);
   }
@@ -386,20 +483,34 @@ void on_run_clicked (GtkWidget *run_control_frame, gpointer user_data) {
 
   /* Sensitize and desensitize various widgets */
   gtk_widget_set_sensitive
-    (glade_xml_get_widget (xml, "options_notebook"), FALSE);
+    (glade_xml_get_widget (xml, "model_options_vbox"), FALSE);
   gtk_widget_set_sensitive
-    (glade_xml_get_widget (xml, "load"), FALSE);
+    (glade_xml_get_widget (xml, "petsc_options_scrolledwindow"), FALSE);
+  gtk_widget_set_sensitive
+    (glade_xml_get_widget (xml, "misc_options_vbox"), FALSE);
+  gtk_widget_set_sensitive
+    (glade_xml_get_widget (xml, "open"), FALSE);
   gtk_widget_set_sensitive
     (glade_xml_get_widget (xml, "save"), FALSE);
+  gtk_widget_set_sensitive
+    (glade_xml_get_widget (xml, "save_as"), FALSE);
   gtk_widget_set_sensitive
     (glade_xml_get_widget (xml, "run"), FALSE);
   gtk_widget_set_sensitive
+    (glade_xml_get_widget (xml, "open_button"), FALSE);
+  gtk_widget_set_sensitive
+    (glade_xml_get_widget (xml, "save_button"), FALSE);
+  gtk_widget_set_sensitive
+    (glade_xml_get_widget (xml, "save_as_button"), FALSE);
+  gtk_widget_set_sensitive
+    (glade_xml_get_widget (xml, "run_button"), FALSE);
+  /* gtk_widget_set_sensitive
     (glade_xml_get_widget (xml, "pause"), TRUE);
   gtk_widget_set_sensitive
-    (glade_xml_get_widget (xml, "stop"), TRUE);
+  (glade_xml_get_widget (xml, "stop"), TRUE); */
 
   /* Show the run control table and text output window */
-  /* gtk_widget_show (run_control_frame); */
+  /* gtk_widget_show (output_window); */
   if (!(gtk_toggle_button_get_active
 	(GTK_TOGGLE_BUTTON (glade_xml_get_widget (xml, "show_output"))))) {
     gtk_toggle_button_set_active
@@ -412,7 +523,7 @@ void on_run_clicked (GtkWidget *run_control_frame, gpointer user_data) {
 				  read_simulation_data, NULL);
 }
 
-void on_stop_clicked (GtkWidget *run_control_frame, gpointer user_data) {
+void on_stop_clicked (GtkWidget *output_window, gpointer user_data) {
   /* This should really send some kind of "stop" signal to chts and wait for a
      reply... interactive options setting, etc. */
 
@@ -424,22 +535,37 @@ void on_stop_clicked (GtkWidget *run_control_frame, gpointer user_data) {
   close (to_simulation_pipe [1]);
 
   /* Make appropriate widget readjustments */
-  /* gtk_widget_hide (run_control_frame); */
+  /* gtk_widget_hide (output_window); */
   gtk_widget_set_sensitive
-    (glade_xml_get_widget (xml, "options_notebook"), TRUE);
+    (glade_xml_get_widget (xml, "model_options_vbox"), TRUE);
   gtk_widget_set_sensitive
-    (glade_xml_get_widget (xml, "load"), TRUE);
+    (glade_xml_get_widget (xml, "petsc_options_scrolledwindow"), TRUE);
+  gtk_widget_set_sensitive
+    (glade_xml_get_widget (xml, "misc_options_vbox"), TRUE);
+  gtk_widget_set_sensitive
+    (glade_xml_get_widget (xml, "open"), TRUE);
   gtk_widget_set_sensitive
     (glade_xml_get_widget (xml, "save"), TRUE);
+  gtk_widget_set_sensitive
+    (glade_xml_get_widget (xml, "save_as"), TRUE);
   gtk_widget_set_sensitive
     (glade_xml_get_widget (xml, "run"), TRUE);
   gtk_widget_set_sensitive
+    (glade_xml_get_widget (xml, "open_button"), TRUE);
+  gtk_widget_set_sensitive
+    (glade_xml_get_widget (xml, "save_button"), TRUE);
+  gtk_widget_set_sensitive
+    (glade_xml_get_widget (xml, "save_as_button"), TRUE);
+  gtk_widget_set_sensitive
+    (glade_xml_get_widget (xml, "run_button"), TRUE);
+  /* gtk_widget_set_sensitive
     (glade_xml_get_widget (xml, "pause"), FALSE);
   gtk_widget_set_sensitive
-    (glade_xml_get_widget (xml, "stop"), FALSE); }
+  (glade_xml_get_widget (xml, "stop"), FALSE); */}
 
 /* Does nothing for now */
 void on_pause_clicked (GtkWidget *forgot, gpointer user_data) {
+  DPRINTF ("Pause was clicked\n");
   return; }
 
 /* And finally, main() */
@@ -450,18 +576,17 @@ int main (int argc, char *argv[])
     *dimension_options, *dimension_menu;
 
   /* Basic init stufff */
-  gtk_init(&argc, &argv);
-  glade_init();
+  gnome_program_init ("CHUI", VERSION, LIBGNOMEUI_MODULE, argc, argv, NULL);
 
   /* Load the interface, display the initial window, and connect the signals */
-  strcpy (buff, CHUI_GLADE_DIRECTORY);
+  strncpy (buff, GLADE_DIRECTORY, 187);
   strcat (buff, "/chui.glade");
-  xml = glade_xml_new (buff, NULL);
+  xml = glade_xml_new (buff, NULL, NULL);
   glade_xml_signal_autoconnect (xml);
 
   /* For some reason, option menus don't quite work in Glade, so I had to make
      a popup menu and attach it to the option menu */
-  transport_options = glade_xml_get_widget (xml, "transport_options");
+  /* transport_options = glade_xml_get_widget (xml, "transport_options");
   gtk_option_menu_remove_menu (GTK_OPTION_MENU (transport_options));
   transport_menu = glade_xml_get_widget (xml, "transport_menu");
   gtk_option_menu_set_menu (GTK_OPTION_MENU(transport_options),transport_menu);
@@ -470,7 +595,7 @@ int main (int argc, char *argv[])
   gtk_option_menu_remove_menu (GTK_OPTION_MENU (dimension_options));
   dimension_menu = glade_xml_get_widget (xml, "dimension_menu");
   gtk_option_menu_set_menu (GTK_OPTION_MENU(dimension_options),dimension_menu);
-  gtk_widget_show (dimension_menu);
+  gtk_widget_show (dimension_menu); */
 
   /* Call handlers to set initial values from Glade defaults */
   on_width_changed (glade_xml_get_widget (xml, "width"), NULL);
