@@ -1,5 +1,5 @@
 /***************************************
-  $Header: /cvsroot/petscgraphics/illuminator.c,v 1.7 2003/05/06 23:15:25 hazelsct Exp $
+  $Header: /cvsroot/petscgraphics/illuminator.c,v 1.9 2004/08/17 15:05:56 hazelsct Exp $
 
   This is the illuminator.c main file.  It has all of the routines which
   compute the triangulation in a distributed way.
@@ -87,9 +87,9 @@ static inline int storetri
 /*++++++++++++++++++++++++++++++++++++++
   This function calculates triangle vertices for an isoquant surface in a
   linear tetrahedron, using the whichplane information supplied by the routine
-  calling this one, and "draws" them using storetri.  This is really an
+  calling this one, and "draws" them using storetri().  This is really an
   internal function, not intended to be called by user programs.  It is used by
-  DrawTet and DrawHex.
+  DrawTet() and DrawHex().
 
   int DrawTetWithPlane Returns 0 or an error code.
 
