@@ -1,5 +1,5 @@
 /***************************************
-  $Header: /cvsroot/petscgraphics/illumulti.c,v 1.26 2004/05/21 22:07:42 hazelsct Exp $
+  $Header: /cvsroot/petscgraphics/illumulti.c,v 1.27 2004/06/30 15:38:18 hazelsct Exp $
 
   This file contains the functions
   +latex+{\tt IlluMultiSave()}, {\tt IlluMultiLoad()} and {\tt IlluMultiRead()}
@@ -469,15 +469,15 @@ static int IlluMultiParseXML
 	  (*fieldnames) [field] = xmlGetProp (thenode, "name");
 	  /*+ If the type attribute is missing from the Field node (as it is in
 	    version 0.1 documents), assume
-	    +latex+{\tt FIELD\_SCALAR\_COLORS}.
-	    +html+ <tt>FIELD_SCALAR_COLORS</tt>. +*/
+	    +latex+{\tt FIELD\_SCALAR}.
+	    +html+ <tt>FIELD_SCALAR</tt>. +*/
 	  buffer = xmlGetProp (thenode, "type");
 	  if (fieldtypes)
 	    {
 	      if (buffer)
 		sscanf (buffer, "0x%x", *fieldtypes + field);
 	      else
-		(*fieldtypes) [field] = FIELD_SCALAR_COLORS;
+		(*fieldtypes) [field] = FIELD_SCALAR;
 	    }
 	  if (buffer)
 	    free(buffer);
