@@ -1,5 +1,5 @@
 /***************************************
-  $Header: /cvsroot/petscgraphics/petsc.c,v 1.9 2004/03/07 01:56:18 hazelsct Exp $
+  $Header: /cvsroot/petscgraphics/petsc.c,v 1.12 2004/07/02 20:51:08 hazelsct Exp $
 
   This is the petsc.c main file.  It has all of the PETSc-dependent functions.
 ***************************************/
@@ -9,9 +9,11 @@
 #include "config.h" /* esp. for inline */
 #include "illuminator.h" /* Just to make sure the interface is "right" */
 
+extern int num_triangles;
+
 
 #undef __FUNCT__
-#define __FUNCT__ "DATriangulate"
+#define __FUNCT__ "DATriangulateRange"
 
 /*++++++++++++++++++++++++++++++++++++++
   Calculate vertices of isoquant triangles in a 3-D distributed array.  This
@@ -23,7 +25,7 @@
   necessary for using default isoquant values, since we need to be able to
   calculate the maximum and minimum on the global array.
 
-  int DATriangulate Returns 0 or an error code.
+  int DATriangulateRange Returns 0 or an error code.
 
   DA theda The PETSc distributed array object.
 
@@ -49,26 +51,26 @@
   +latex+{\tt PETSC\_NULL} if {\tt n\_quants=PETSC\_DECIDE}.
   +html+ <tt>PETSC_NULL</tt> if <tt>n_quants=PETSC\_DECIDE</tt>.
 
-  PetscTruth xcut If
-  +latex+{\tt PETSC\_TRUE}, leave out the final $x$
-  +html+ <tt>PETSC_TRUE</tt>, leave out the final <i>x</i>
-  row/plane in spite of periodicity.
+  int xmin Smallest grid x-coordinate to render.
 
-  PetscTruth ycut If
-  +latex+{\tt PETSC\_TRUE}, leave out the final $y$
-  +html+ <tt>PETSC_TRUE</tt>, leave out the final <i>y</i>
-  row/plane in spite of periodicity.
+  int xmax Largest grid x-coordinate to render, -1 goes to full x maximum, -2
+  in periodic systems goes to one short of x maximum.
 
-  PetscTruth zcut If
-  +latex+{\tt PETSC\_TRUE}, leave out the final $z$
-  +html+ <tt>PETSC_TRUE</tt>, leave out the final <i>z</i>
-  row/plane in spite of periodicity.
+  int ymin Smallest grid y-coordinate to render.
+
+  int ymax Largest grid y-coordinate to render, -1 goes to full y maximum, -2
+  in periodic systems goes to one short of y maximum.
+
+  int zmin Smallest grid z-coordinate to render.
+
+  int zmax Largest grid z-coordinate to render, -1 goes to full z maximum, -2
+  in periodic systems goes to one short of z maximum.
   ++++++++++++++++++++++++++++++++++++++*/
 
-int DATriangulate
+int DATriangulateRange
 (DA theda, Vec globalX, int this, PetscScalar *minmax, int n_quants,
- PetscScalar *isoquants, PetscScalar *colors, PetscTruth xcut, PetscTruth ycut,
- PetscTruth zcut)
+ PetscScalar *isoquants, PetscScalar *colors, int xmin,int xmax, int ymin,
+ int ymax, int zmin,int zmax)
 {
   Vec localX;
   PetscScalar isoquantdefaults[4],
@@ -95,9 +97,10 @@ int DATriangulate
   ierr = DAGlobalToLocalBegin (theda, globalX, INSERT_VALUES, localX); CHKERRQ(ierr);
   ierr = DAGlobalToLocalEnd (theda, globalX, INSERT_VALUES, localX); CHKERRQ (ierr);
 
-  /* Use DATriangulateLocal() to do the work */
-  ierr = DATriangulateLocal (theda, localX, this, minmax, n_quants, isoquants,
-			     colors, xcut, ycut, zcut); CHKERRQ (ierr);
+  /* Use DATriangulateLocalRange() to do the work */
+  ierr = DATriangulateLocalRange (theda, localX, this, minmax, n_quants,
+				  isoquants, colors, xmin,xmax, ymin,ymax,
+				  zmin,zmax); CHKERRQ (ierr);
 
   ierr = VecDestroy (localX); CHKERRQ (ierr);
 
@@ -106,14 +109,14 @@ int DATriangulate
 
 
 #undef __FUNCT__
-#define __FUNCT__ "DATriangulateLocal"
+#define __FUNCT__ "DATriangulateLocalRange"
 
 /*++++++++++++++++++++++++++++++++++++++
   Calculate vertices of isoquant triangles in a 3-D distributed array.  This
   takes a PETSc DA object, does some sanity checks, calculates array sizes, and
   then gets array and passes it to Draw3DBlock for triangulation.
 
-  int DATriangulateLocal Returns 0 or an error code.
+  int DATriangulateLocalRange Returns 0 or an error code.
 
   DA theda The PETSc distributed array object.
 
@@ -136,26 +139,26 @@ int DATriangulate
 
   PetscScalar *colors Array of color R,G,B,A quads for each isoquant.
 
-  PetscTruth xcut If
-  +latex+{\tt PETSC\_TRUE}, leave out the final $x$
-  +html+ <tt>PETSC_TRUE</tt>, leave out the final <i>x</i>
-  row/plane in spite of periodicity.
+  int xmin Smallest grid x-coordinate to render.
 
-  PetscTruth ycut If
-  +latex+{\tt PETSC\_TRUE}, leave out the final $y$
-  +html+ <tt>PETSC_TRUE</tt>, leave out the final <i>y</i>
-  row/plane in spite of periodicity.
+  int xmax Largest grid x-coordinate to render, -1 goes to full x maximum, -2
+  in periodic systems goes to one short of x maximum.
 
-  PetscTruth zcut If
-  +latex+{\tt PETSC\_TRUE}, leave out the final $z$
-  +html+ <tt>PETSC_TRUE</tt>, leave out the final <i>z</i>
-  row/plane in spite of periodicity.
+  int ymin Smallest grid y-coordinate to render.
+
+  int ymax Largest grid y-coordinate to render, -1 goes to full y maximum, -2
+  in periodic systems goes to one short of y maximum.
+
+  int zmin Smallest grid z-coordinate to render.
+
+  int zmax Largest grid z-coordinate to render, -1 goes to full z maximum, -2
+  in periodic systems goes to one short of z maximum.
   ++++++++++++++++++++++++++++++++++++++*/
 
-int DATriangulateLocal
+int DATriangulateLocalRange
 (DA theda, Vec localX, int this, PetscScalar *minmax, int n_quants,
- PetscScalar *isoquants, PetscScalar *colors, PetscTruth xcut, PetscTruth ycut,
- PetscTruth zcut)
+ PetscScalar *isoquants, PetscScalar *colors, int xmin,int xmax, int ymin,
+ int ymax, int zmin,int zmax)
 {
   PetscScalar *x, isoquantdefaults[4], localminmax[6],
     colordefaults[16] = { 1,0,0,.5, 1,1,0,.5, 0,1,0,.5, 0,0,1,.5 };
@@ -164,21 +167,16 @@ int DATriangulateLocal
 
   /* Default isoquant error */
   if (n_quants == PETSC_DECIDE)
-    {
-      SETERRQ (PETSC_ERR_ARG_WRONGSTATE, "Can't get default isoquants for local array");
-    }
+    SETERRQ (PETSC_ERR_ARG_WRONGSTATE, "Can't get default isoquants for local array");
 
   /* Get global and local grid boundaries */
   ierr = DAGetInfo (theda, &i, &xw,&yw,&zw, PETSC_NULL,PETSC_NULL,PETSC_NULL,
 		    &fields, PETSC_NULL, PETSC_NULL, &stencil);CHKERRQ(ierr);
   if (i!=3)
-    {
-      SETERRQ (PETSC_ERR_ARG_WRONGSTATE, "DA must be 3-dimensional");
-    }
+    SETERRQ (PETSC_ERR_ARG_WRONGSTATE, "DA must be 3-dimensional");
   if (stencil!=DA_STENCIL_BOX)
-    {
-      SETERRQ (PETSC_ERR_ARG_WRONGSTATE, "DA must have a box stencil");
-    }
+    SETERRQ (PETSC_ERR_ARG_WRONGSTATE, "DA must have a box stencil");
+
   ierr = DAGetCorners (theda, &xs,&ys,&zs, &xm,&ym,&zm); CHKERRQ (ierr);
   ierr = DAGetGhostCorners (theda, &gxs,&gys,&gzs, &gxm,&gym,&gzm);
   CHKERRQ (ierr);
@@ -186,14 +184,6 @@ int DATriangulateLocal
   /* Get the local array of points, with ghosts */
   ierr = VecGetArray (localX, &x); CHKERRQ (ierr);
 
-  /* Calculate local physical size */
-  localminmax[0] = minmax[0] + (minmax[1]-minmax[0])*xs/xw;
-  localminmax[1] = minmax[0] + (minmax[1]-minmax[0])*(xs+xm)/xw;
-  localminmax[2] = minmax[2] + (minmax[3]-minmax[2])*ys/yw;
-  localminmax[3] = minmax[2] + (minmax[3]-minmax[2])*(ys+ym)/yw;
-  localminmax[4] = minmax[4] + (minmax[5]-minmax[4])*zs/zw;
-  localminmax[5] = minmax[4] + (minmax[5]-minmax[4])*(zs+zm)/zw;
-
   /* If the array is too big, cut it down to size */
   if (gxm <= xs-gxs+xm)
     xm = gxm-xs+gxs-1;
@@ -202,22 +192,38 @@ int DATriangulateLocal
   if (gzm <= zs-gzs+zm)
     zm = gzm-zs+gzs-1;
   /* Eliminate final rows/planes if *cut and periodic. */
-  if (xcut && xs+xm>=xw)
+  if (xmax == -2 && xs+xm>=xw)
     xm--;
-  if (ycut && ys+ym>=yw)
+  if (ymax == -2 && ys+ym>=yw)
     ym--;
-  if (zcut && zs+zm>=zw)
+  if (zmax == -2 && zs+zm>=zw)
     zm--;
 
+  /* Reframe to range */
+  xs = PetscMax (xs, xmin);
+  ys = PetscMax (ys, xmin);
+  zs = PetscMax (zs, xmin);
+  xm = (xmax > 0) ? PetscMin (xm, xmax-xs) : xm;
+  ym = (ymax > 0) ? PetscMin (ym, ymax-ys) : ym;
+  zm = (zmax > 0) ? PetscMin (zm, zmax-zs) : zm;
+
+  /* Calculate local physical size */
+  localminmax[0] = minmax[0] + (minmax[1]-minmax[0])*xs/xw;
+  localminmax[1] = minmax[2] + (minmax[1]-minmax[0])*(xs+xm)/xw;
+  localminmax[2] = minmax[2] + (minmax[3]-minmax[2])*ys/yw;
+  localminmax[3] = minmax[2] + (minmax[3]-minmax[2])*(ys+ym)/yw;
+  localminmax[4] = minmax[4] + (minmax[5]-minmax[4])*zs/zw;
+  localminmax[5] = minmax[4] + (minmax[5]-minmax[4])*(zs+zm)/zw;
+
   /* Let 'er rip! */
   ierr = Draw3DBlock (gxm,gym,gzm, xs-gxs,ys-gys,zs-gzs, xm,ym,zm, localminmax,
 		      x+this, fields, n_quants, isoquants, colors);
   CHKERRQ (ierr);
   ierr = VecRestoreArray (localX, &x); CHKERRQ (ierr);
 
-  /* This debugging information may be useful to someone at some point;
-     note that it requires "extern int num_triangles;" somewhere above. */
-  /* {
+  /* Prints the number of triangles on all CPUs */
+#ifdef DEBUG
+  {
     int rank;
     ierr = MPI_Comm_rank (PETSC_COMM_WORLD, &rank); CHKERRQ (ierr);
     ierr = PetscSynchronizedPrintf
@@ -227,7 +233,8 @@ int DATriangulateLocal
     ierr = PetscSynchronizedPrintf (PETSC_COMM_WORLD, "[%d] Triangles: %d\n",
 				    rank, num_triangles); CHKERRQ (ierr);
     ierr = PetscSynchronizedFlush (PETSC_COMM_WORLD); CHKERRQ (ierr);
-    } */
+  }
+#endif
 
   return 0;
 }
