@@ -1,5 +1,5 @@
 /***************************************
-  $Header: /cvsroot/petscgraphics/tsview-ng.c,v 1.10 2004/05/21 22:08:44 hazelsct Exp $
+  $Header: /cvsroot/petscgraphics/tsview-ng.c,v 1.21 2004/08/11 12:44:08 hazelsct Exp $
 
   This program views the output of a time series saved using
   +latex+{\tt IlluMultiSave()}.
@@ -53,183 +53,19 @@ DA theda;
 Vec global;
 PetscScalar minmax[6] = { 0.,1., 0.,1., 0.,1. };
 field_plot_type *fieldtypes;
-int dimensions, num_fields, num_variables[1], **variable_indices;
+int dimensions, num_fields, current_field, *field_index, num_variables[1],
+  **variable_indices;
 
-/* First some main functions which do stuff, then callbacks, then main(). */
-static inline void pseudocolor (PetscScalar val, PetscScalar min,
-				   PetscScalar max, guchar *pixel)
-{
-  PetscScalar shade = (val - min) / (max - min);
-  /* Old stuff *
-  if (shade < 0.2) {      /* red -> yellow *
-    *pixel++ = 255;
-    *pixel++ = 1275*shade;
-    *pixel++ = 0; }
-  else if (shade < 0.4) { /* yellow -> green *
-    *pixel++ = 510-1275*shade;
-    *pixel++ = 255;
-    *pixel++ = 0; }
-  else if (shade < 0.6) { /* green -> cyan *
-    *pixel++ = 0;
-    *pixel++ = 255;
-    *pixel++ = 1275*shade-510; }
-  else if (shade < 0.8) { /* cyan -> blue *
-    *pixel++ = 0;
-    *pixel++ = 1020-1275*shade;
-    *pixel++ = 255; }
-  else {                  /* blue -> magenta *
-    *pixel++ = 1275*shade-1020;
-    *pixel++ = 0;
-    *pixel++ = 255; }
-  /* New stuff */
-  if (shade < 0.25) {      /* red -> yellow */
-    *pixel++ = 255;
-    *pixel++ = 1020*shade;
-    *pixel++ = 0; }
-  else if (shade < 0.5) { /* yellow -> green */
-    *pixel++ = 510-1020*shade;
-    *pixel++ = 255;
-    *pixel++ = 0; }
-  else if (shade < 0.75) { /* green -> cyan */
-    *pixel++ = 0;
-    *pixel++ = 255;
-    *pixel++ = 1020*shade-510; }
-  else { /* cyan -> blue */
-    *pixel++ = 0;
-    *pixel++ = 1020-1020*shade;
-    *pixel++ = 255; }
-}
+/* First some primary functions which do stuff, then callbacks, then main(). */
 
-static inline void pseudohueintcolor
-(PetscScalar vx, PetscScalar vy, PetscScalar refmag, guchar *pixel)
-{
-  PetscScalar mag=sqrt(vx*vx+vy*vy), theta=atan2(vy,vx), red, green, blue;
-  if (refmag <= 0.)
-    {
-      *pixel = *(pixel+1) = *(pixel+2) = 0;
-      return;
-    }
-  mag = (mag > refmag) ? 1.0 : mag/refmag;
-  /* Old pseudocolor */
-  /* red = (theta<-M_PI*5./6.)? 1.5/M_PI*(-M_PI*5./6.-theta):
-    ((theta<-M_PI/6.)? 0.:
-     ((theta<M_PI/2.)? 1.5/M_PI*(theta+M_PI/6.): 1.5/M_PI*(M_PI*7./6.-theta)));
-  green = (theta<-M_PI*5./6.)? 1.5/M_PI*(theta+M_PI*3./2.):
-    ((theta<-M_PI/6.)? 1.5/M_PI*(-M_PI/6.-theta):
-     ((theta<M_PI/2.)? 0.: 1.5/M_PI*(theta-M_PI/2.)));
-  blue = (theta<-M_PI*5./6.)? 0.:
-    ((theta<-M_PI/6.)? 1.5/M_PI*(theta+M_PI*5./6.):
-    ((theta<M_PI/2.)? 1.5/M_PI*(M_PI/2.-theta): 0.)); */
-
-  /* New pseudocolor: green, yellow, red, blue at 0, 90, 180, 270 degrees */
-  red = 2./M_PI * ((theta<-M_PI/2.) ? -M_PI/2.-theta :
-		   ((theta<0.) ? 0. : ((theta<M_PI/2.) ? theta : M_PI/2.)));
-  green = 2./M_PI * ((theta<-M_PI/2.) ? 0. :
-		     ((theta<0.) ? theta+M_PI/2. :
-		      ((theta<M_PI/2.) ? M_PI/2. : M_PI-theta)));
-  blue = 2./M_PI * ((theta<-M_PI/2.) ? theta+M_PI :
-		    ((theta<0.) ? -theta : 0.));
-
-  *pixel++ = 255*mag*red;
-  *pixel++ = 255*mag*green;
-  *pixel++ = 255*mag*blue;
-}
-
-static inline void pseudoternarycolor
-(PetscScalar A, PetscScalar B, PetscScalar Corner1A, PetscScalar Corner1B,
- PetscScalar Corner2A, PetscScalar Corner2B, PetscScalar Corner3A,
- PetscScalar Corner3B, guchar *pixel)
-{
-  PetscScalar x1, x2, inverse_det;
-
-  /* Transform A,B into x1,x2 based on corners */
-  A        -= Corner3A;
-  Corner1A -= Corner3A;
-  Corner2A -= Corner3A;
-  B        -= Corner3B;
-  Corner1B -= Corner3B;
-  Corner2B -= Corner3B;
-  inverse_det = 1./(Corner1A*Corner2B - Corner2A*Corner1B);
-  x1 = (A*Corner2B - B*Corner2A) * inverse_det;
-  x2 = (B*Corner1A - A*Corner1B) * inverse_det;
-
-  /* Now colorize */
-  *pixel++ = 255*(1.-x1);
-  *pixel++ = 255*(x1+x2);
-  *pixel++ = 255*(1.-x2);
-}
-
-/* Find the maximum magnitude of a 2-D vector field in an array */
-PetscScalar ArrayMaxMag (PetscScalar *global, int points, int fields,int field)
-{
-  int i;
-  PetscScalar retval;
-
-  retval = global[field]*global[field] + global[field+1]*global[field+1];
-
-  for (i=1; i<points; i++)
-    retval = PetscMax (retval, global[i*fields+field]*global[i*fields+field] +
-		       global[i*fields+field+1]*global[i*fields+field+1]);
-  return sqrt (retval);
-}
-
-void paint_rgb (guchar *rgb, int rwidth, int rheight, DA rgbda, Vec rgbglobal,
-		int display_field)
-{
-  int nx,ny,nz, ix,iy, ierr;
-  PetscScalar *global_array, minval, maxval, refmag;
-
-  ierr = DAGetInfo (rgbda, PETSC_NULL, &nx,&ny,&nz,
-		    PETSC_NULL,PETSC_NULL,PETSC_NULL, PETSC_NULL,
-		    PETSC_NULL,PETSC_NULL,PETSC_NULL); CHKERRQ (ierr);
-
-  ierr = VecStrideMin (rgbglobal, display_field, PETSC_NULL, &minval);
-  CHKERRQ (ierr);
-  ierr = VecStrideMax (rgbglobal, display_field, PETSC_NULL, &maxval);
-  CHKERRQ (ierr);
-
-  ierr = VecGetArray (rgbglobal, &global_array);
-
-  /* If a vector field, then find max magnitude in fields 0-1 */
-  if (fieldtypes[display_field] == FIELD_VECTOR_HUEINTENSE ||
-      fieldtypes[display_field] == FIELD_VECTOR_ARROWS)
-    {
-      if (vecmax > 0)
-	refmag = vecmax;
-      else
-	refmag = ArrayMaxMag (global_array, nx*ny, num_fields, display_field);
-      printf ("Reference vector magnitude = %g\n", refmag);
-    }
-
-  for (iy=0; iy<rheight; iy++)
-    for (ix=0; ix<rwidth; ix++)
-      {
-	int vecindex;
-	guchar *pixel;
-	vecindex = (((rheight-iy-1)*ny/rheight)*nx+ ix*nx/rwidth)*num_fields+
-	  display_field;
-	pixel = rgb + 3*(iy*rwidth + ix);
-
-	if (fieldtypes[display_field] == FIELD_TERNARY_RGB)
-	  pseudoternarycolor
-	    (global_array [vecindex], global_array [vecindex+1],
-	     0.,0., 1.,0., 0.,1., pixel);
-	else if (fieldtypes[display_field] == FIELD_VECTOR_HUEINTENSE ||
-		 fieldtypes[display_field] == FIELD_VECTOR_ARROWS)
-	  pseudohueintcolor
-	    (global_array [vecindex], global_array [vecindex+1], refmag,
-	     pixel);
-	else
-	  pseudocolor (global_array [vecindex], minval, maxval, pixel);
-      }
-  ierr = VecRestoreArray (rgbglobal, &global_array);
-}
+#undef __FUNCT__
+#define __FUNCT__ "render_dataviews"
 
 void render_dataviews ()
 {
   int viewnum, nx,ny,nz, ierr;
-  GtkWidget *thebar = glade_xml_get_widget(xml, "appbar1");
 
+  DPRINTF ("Rendering dataviews\n",0);
   if (dataview_count != 1)
     {
       printf ("dataview_count != 1 is not yet supported\n");
@@ -237,25 +73,36 @@ void render_dataviews ()
     }
   for (viewnum=0; viewnum<dataview_count; viewnum++)
     {
-      int ix,iy, display_field=0;
+      int nx,ny, xs,ys, xm,ym, ix,iy;
       PetscScalar minval, maxval, refmag, *global_array;
       GtkType type;
       char thestatus [100];
 
       /* (Re)allocate buffer */
+      DPRINTF ("(Re)allocating RGB buffer\n",0);
       if (!(rgbbuf [viewnum] =
 	    (guchar *) realloc (rgbbuf [viewnum],
 				3*width*height*sizeof(guchar)))) {
 	  printf ("ERROR: can't reallocate RGB buffer\n");
 	  exit (1); }
 
-      sprintf (thestatus, "Rendering view %d...", viewnum);
-      gnome_appbar_set_default (GNOME_APPBAR (thebar), thestatus);
-
-      /* Render into rgbbuf */
-      paint_rgb (rgbbuf [viewnum], width,height, theda, global, display_field);
+      /* Render into rgbbuf [viewnum] */
+      ierr = DAGetInfo (theda, PETSC_NULL, &nx,&ny,PETSC_NULL,
+			PETSC_NULL,PETSC_NULL,PETSC_NULL, PETSC_NULL,
+			PETSC_NULL,PETSC_NULL,PETSC_NULL); CHKERRQ (ierr);
+      ierr = DAGetCorners (theda, &xs,&ys,PETSC_NULL, &xm,&ym,PETSC_NULL);
+      CHKERRQ (ierr);
+      ierr = VecGetArray (global, &global_array); CHKERRQ (ierr);
+      ierr = render_rgb_local_2d
+	(rgbbuf [viewnum], width,height, 3, global_array, num_fields,
+	 current_field, fieldtypes [current_field],
+	 ((fieldtypes [current_field] == FIELD_VECTOR ||
+	   fieldtypes [current_field] == FIELD_VECTOR) &&
+	  vecmax > 0) ? (&vecmax)-1 : NULL, nx,ny, xs,ys, xm,ym);
+      ierr = VecRestoreArray (global, &global_array); CHKERRQ (ierr);
 
       /* Draw buffer onto window */
+      DPRINTF ("Painting rgb buffer onto window\n",0);
       if (!dataviews [viewnum])
 	dataviews [viewnum] = glade_xml_get_widget (xml, "plot_area");
       gtk_drawing_area_size (GTK_DRAWING_AREA (dataviews [viewnum]),
@@ -265,7 +112,6 @@ void render_dataviews ()
 			  0, 0, width, height, GDK_RGB_DITHER_MAX,
 			  rgbbuf [viewnum], width * 3);
     }
-  gnome_appbar_set_default (GNOME_APPBAR (thebar), "Done.");
 }
 
 
@@ -336,8 +182,7 @@ int refresh_stepnames ()
       ierr = PetscFinalize (); CHKERRQ(ierr);
       return 1;
     }
-  ierr = PetscPrintf (PETSC_COMM_WORLD, "%d eligible files:\n", total_entries);
-  CHKERRQ (ierr);
+  DPRINTF ("%d eligible files:\n", total_entries);
 
   if (!(stepnames = (char **) realloc
 	(stepnames, total_entries*sizeof (char *))))
@@ -354,7 +199,7 @@ int refresh_stepnames ()
       strncpy (stepnames [i], namelist[i]->d_name, filength-13);
       stepnames [i] [filength-13] = '\0';
       free (namelist[i]);
-      ierr = PetscPrintf (PETSC_COMM_WORLD, "[%d] %s\n", i, stepnames [i]);
+      DPRINTF ("[%d] %s\n", i, stepnames [i]);
       CHKERRQ (ierr);
     }
 
@@ -362,6 +207,12 @@ int refresh_stepnames ()
   return 0;
 }
 
+void change_variable (GtkWidget *widget, gpointer user_data)
+{
+  current_field = GPOINTER_TO_INT (widget);
+  DPRINTF ("Switching to variable %d\n", current_field);
+  render_dataviews();
+}
 
 void on_mag_spin_value_changed (GtkWidget *mag_spin, gpointer user_data) {
   G_CONST_RETURN gchar *entrytext;
@@ -370,7 +221,7 @@ void on_mag_spin_value_changed (GtkWidget *mag_spin, gpointer user_data) {
   width = (int) (minmax [1] * sizemag);
   height = (int) (minmax [3] * sizemag);
 
-  render_dataviews(dataviews, dataview_count);
+  render_dataviews();
 }
 
 
@@ -401,34 +252,30 @@ void display_timestep (int usermetacount, char **usermetanames,
 void on_save_activate (GtkWidget *widget, gpointer user_data)
 {
   int i, ierr;
-  char **usermetanames, **usermetadata, filename [200], thestatus [200];
-  GtkWidget *thebar = glade_xml_get_widget(xml, "appbar1");
+  char **usermetanames, **usermetadata, filename [200], number[10];
   FILE *outppm;
 
   strncpy (filename, basedirname, 198);
   strcat (filename, "/");
   strncat (filename, stepnames [entrynum], 198 - strlen (filename));
+  snprintf (number, 9, "-f%d", current_field);
+  strncat (filename, number, 198 - strlen (filename));
   strncat (filename, ".ppm", 198 - strlen (filename));
 
-  sprintf (thestatus, "Saving entry %d in filename %s", entrynum, filename);
-  gnome_appbar_set_default (GNOME_APPBAR (thebar), thestatus);
-
+  DPRINTF ("Saving image with filename %s\n", filename);
   if (!(outppm = fopen (filename, "w")))
     printf ("Error opening file %s\n", filename);
   fprintf (outppm, "P6\n%d %d\n255\n", width, height);
   fwrite (rgbbuf [0], sizeof (guchar), 3*width*height, outppm);
   fclose (outppm);
-
-  sprintf (thestatus, "Saved entry %d in filename %s", entrynum, filename);
-  gnome_appbar_set_default (GNOME_APPBAR (thebar), thestatus);
 }
 
 
 void on_timestep_spin_value_changed (GtkWidget *timestep_spin, gpointer user_data) {
   int usermetacount, ierr;
   G_CONST_RETURN gchar *entrytext;
-  char **usermetanames, **usermetadata, filename [200], thestatus [200];
-  GtkWidget *thebar = glade_xml_get_widget(xml, "appbar1");
+  char **usermetanames, **usermetadata, filename [200], **field_name;
+  GtkWidget *variable_options, *variable_menu, **variable_item;
 
   entrytext = gtk_entry_get_text (GTK_ENTRY (timestep_spin));
   sscanf (entrytext, "%d", &entrynum);
@@ -446,9 +293,6 @@ void on_timestep_spin_value_changed (GtkWidget *timestep_spin, gpointer user_dat
   strcat (filename, "/");
   strncat (filename, stepnames [entrynum], 198 - strlen (filename));
 
-  sprintf (thestatus, "Loading entry %d, basename %s\n", entrynum, filename);
-  gnome_appbar_set_default (GNOME_APPBAR (thebar), thestatus);
-
   ierr = IlluMultiRead (theda, global, filename, &usermetacount,&usermetanames,
 			&usermetadata); CHKERRQ (ierr);
 
@@ -464,187 +308,6 @@ void on_about_activate (GtkWidget *none, gpointer user_data) {
   gtk_widget_show (glade_xml_get_widget (xml, "about")); }
 
 
-/*
-#define HELP_STRING "tsview commands:\n\
-  <enter> Display next timestep\n\
-  b       Display previous timestep\n\
-  ###     Jump to timestep ###\n\
-  t       Toggle Geomview transparency (3-D only)\n\
-  v       Change field displayed (3-D only)\n\
-  h/?     Print this information\n\
-  q/x     Quit tsview\n"
-
-int start (int argc, char *argv[])
-{
-  int total_entries, current_entry, dims, i, ierr;
-  PetscViewer theviewer;
-  PetscTruth loaded = PETSC_FALSE, transp=PETSC_TRUE; */
-
-  /*+In the main loop, the various timesteps are displayed, with options:
-    +latex+\begin{itemize} \item
-    +html+ <ul><li>
-    A number jumps to that entry in the files table.
-    +latex+\item {\stt <return>}
-    +html+ <li><tt>&lt;return&gt;</tt>
-    loads the next file.
-    +latex+\item {\tt b}
-    +html+ <li><tt>b</tt>
-    goes back one file.
-    +latex+\item {\tt q}
-    +html+ <li><tt>q</tt>
-    quits the program.
-    +latex+\end{itemize}
-    +html+ </ul>
-    +*/
-  /*  current_entry=0;
-  while (1)
-    {
-
-      else
-	{
-
-	  if (dims<3)
-	    {
-	      int width=300, height=300;
-
-	      if (minmax[1]<minmax[3])
-		width  *= minmax[1]/minmax[3];
-	      else
-		height *= minmax[3]/minmax[1];
-
-	      ierr = PetscViewerDrawOpen
-		(PETSC_COMM_WORLD, 0, "", PETSC_DECIDE, PETSC_DECIDE,
-		 width, height, &theviewer); CHKERRQ (ierr);
-	    }
-	  else
-	    {
-	      ierr = GeomviewBegin (PETSC_COMM_WORLD);
-	    }
-	}
-
-      ierr = PetscPrintf (PETSC_COMM_WORLD, "User data:\n"); CHKERRQ (ierr);
-      for (i=0; i<usermetacount; i++)
-	{
-	  ierr = PetscPrintf (PETSC_COMM_WORLD, "%s = %s\n", usermetanames [i],
-			      usermetadata [i]); CHKERRQ (ierr);
-	}
-
-      if (dims<3)
-	{
-	  ierr = VecView (global, theviewer); CHKERRQ (ierr);
-	}
-      else
-      { */
-	  /*+ The Illuminator-based 3-D viewer can only display one field at a
-	    time.  At the beginning, that is field 0, and is cycled using the
-	    +latex+{\tt v}
-	    +html+ <tt>v</tt>
-	    command. +*/
-	  /* PetscScalar minval, maxval;
-	  char *fieldname;
-
-	  ierr = VecStrideMin (global, display_field, PETSC_NULL, &minval);
-	  CHKERRQ (ierr);
-	  ierr = VecStrideMax (global, display_field, PETSC_NULL, &maxval);
-	  CHKERRQ (ierr);
-	  ierr = PetscPrintf (PETSC_COMM_WORLD,
-			      "Displaying field %d [%g-%g]: %s\n",
-			      display_field, minval, maxval, fieldname);
-	  CHKERRQ (ierr);
-
-	  DPRINTF ("Calculating triangle locations\n",0);
-	  ierr = DATriangulate (theda, global, display_field, minmax,
-				PETSC_DECIDE, PETSC_NULL, PETSC_NULL);
-	  CHKERRQ (ierr);
-	  ierr = DAGetFieldName (theda, display_field, &fieldname);
-	  CHKERRQ (ierr);
-	  DPRINTF ("Consolidating triangles on head node and visualizing\n",0);
-	  ierr = GeomviewDisplayTriangulation
-	    (PETSC_COMM_WORLD, minmax, fieldname, transp);
-	  CHKERRQ (ierr);
-	  }
-
-      for (i=0; i<usermetacount; i++)
-	{
-	  free (usermetanames [i]);
-	  free (usermetadata [i]);
-	}
-      free (usermetanames);
-      free (usermetadata);
-
-      ierr = PetscPrintf (PETSC_COMM_WORLD, "What to do? (h for options) ");
-      CHKERRQ (ierr);
-      ierr = PetscSynchronizedFGets (PETSC_COMM_WORLD, stdin, 99, instring);
-      CHKERRQ (ierr);
-
-      switch (instring [0])
-	{
-	case 'q':
-	case 'Q':
-	case 'x':
-	case 'X':
-	  {
-	    if (dims < 3)
-	      {
-		ierr = PetscViewerDestroy (theviewer); CHKERRQ (ierr);
-	      }
-	    else
-	      {
-		ierr = GeomviewEnd (PETSC_COMM_WORLD); CHKERRQ (ierr);
-	      }
-	    ierr = PetscFinalize(); CHKERRQ (ierr);
-	    return 0;
-	  }
-	case 't':
-	case 'T':
-	  {
-	    transp=!transp;
-	    break;
-	  }
-	case 'h':
-	case 'H':
-	case '?':
-	  {
-	    ierr = PetscPrintf (PETSC_COMM_WORLD, HELP_STRING);
-	    break;
-	  }
-	case '0':
-	case '1':
-	case '2':
-	case '3':
-	case '4':
-	case '5':
-	case '6':
-	case '7':
-	case '8':
-	case '9':
-	  {
-	    current_entry = atoi (instring);
-	    break;
-	  }
-	case 'b':
-	case 'B':
-	  {
-	    current_entry--;
-	    break;
-	  }
-	case 'v':
-	case 'V':
-	  {
-	    if (dims == 3)
-	      display_field = (display_field+1) % fields;
-	  }
-	default:
-	  current_entry++;
-	}
-      if (current_entry < 0)
-	current_entry = total_entries-1;
-      if (current_entry >= total_entries)
-	current_entry = 0;
-    }
-
-    } */
-
 #undef __FUNCT__
 #define __FUNCT__ "main"
 
@@ -667,7 +330,8 @@ int main (int argc, char *argv[])
     { "vector_max", 'vm', POPT_ARG_FLOAT, &vecmax, 0, "Reference vector length", "VECMAX" },
     { NULL, '\0', 0, NULL, 0, NULL, NULL }}; */
   int usermetacount=0, i, ierr;
-  char **usermetanames, **usermetadata, filename [200];
+  char **usermetanames, **usermetadata, filename [200], **field_name;
+  GtkWidget *variable_options, *variable_menu, **variable_item;
 
   /*+ After
     +latex+{\tt PETSc}
@@ -703,6 +367,7 @@ int main (int argc, char *argv[])
      fix: use GNOME arguments instead of PETSc. */
   argc=2;
 
+  DPRINTF ("Running gnome_program_init with argc=2\n",0);
   gnome_program_init ("TSView", VERSION, LIBGNOMEUI_MODULE, argc, argv, NULL);
 
   strncpy (filename, GLADE_DIRECTORY, 187);
@@ -718,6 +383,7 @@ int main (int argc, char *argv[])
       gtk_widget_show (glade_xml_get_widget (xml, "open_fileselect"));
     }
 
+  DPRINTF ("Loading list of timestep names\n",0);
   if (refresh_stepnames ())
     exit (1);
 
@@ -732,6 +398,7 @@ int main (int argc, char *argv[])
 
   /* Usermetadata xwidth, ywidth, zwidth override minmax in case IlluMulti
      version of saved data is 0.1. */
+  DPRINTF ("Checking usermetadata for width information\n",0);
   for (i=0; i<usermetacount; i++)
     {
       if (!strncmp (usermetanames [i], "xwidth", 6))
@@ -742,24 +409,53 @@ int main (int argc, char *argv[])
 	sscanf (usermetadata [i], "%lf", minmax+5);
     }
 
+  DPRINTF ("Getting distributed array global vector and info\n",0);
   ierr = DAGetGlobalVector (theda, &global); CHKERRQ (ierr);
   ierr = DAGetInfo (theda, &dimensions, PETSC_NULL,PETSC_NULL,PETSC_NULL,
 		    PETSC_NULL,PETSC_NULL,PETSC_NULL, &num_fields,
 		    PETSC_NULL,PETSC_NULL,PETSC_NULL); CHKERRQ (ierr);
+  if (dimensions != 2)
+    SETERRQ (PETSC_ERR_ARG_OUTOFRANGE, "tsview-ng only supports 2-D distributed arrays at this point.");
+
+  /* Build menu of field variables */
+  variable_options = glade_xml_get_widget (xml, "variable_menu");
+  gtk_option_menu_remove_menu (GTK_OPTION_MENU (variable_options));
+  variable_menu = gtk_menu_new ();
+  variable_item = (GtkWidget **) malloc (num_fields * sizeof (GtkWidget *));
+  field_name = (char **) malloc (num_fields * sizeof (char *));
+  field_index = (int *) malloc (num_fields * sizeof (int));
+  field_indices (num_fields, dimensions, fieldtypes, field_index);
+  DPRINTF ("Field indices:\n",0);
+  for (i=0; i<num_fields && field_index [i] != -1; i++)
+    {
+      ierr = DAGetFieldName (theda, field_index [i], field_name+i);
+      CHKERRQ (ierr);
+      DPRINTF ("%d index %d name %s\n", i, field_index [i], field_name [i]);
+      variable_item [i] = gtk_menu_item_new_with_label (field_name [i]);
+      gtk_menu_append (GTK_MENU (variable_menu), variable_item [i]);
+      gtk_signal_connect_object (GTK_OBJECT (variable_item [i]), "activate",
+				 GTK_SIGNAL_FUNC (change_variable),
+				 GINT_TO_POINTER (field_index [i]));
+      gtk_widget_show (variable_item [i]);
+    }
+  gtk_option_menu_set_menu (GTK_OPTION_MENU (variable_options), variable_menu);
+  gtk_widget_show (variable_menu);
+  gtk_widget_show (variable_options);
 
   /* Main window title */
   {
     char main_window_title[100] = "TSView: ";
-    strncat (main_window_title, basename (the_basename), 90);
-    gtk_window_set_title
-      (GTK_WINDOW (glade_xml_get_widget (xml, "main_window")),
-       main_window_title);
-  }
-  gtk_widget_set_sensitive (glade_xml_get_widget (xml, "toolbar"), TRUE);
-  gtk_widget_show (glade_xml_get_widget (xml, "vbox1"));
+    GtkWidget *main_window = glade_xml_get_widget (xml, "main_window");
 
+    strncat (main_window_title, basename (the_basename), 90);
+    gtk_window_set_title (GTK_WINDOW (main_window), main_window_title);
+    gtk_widget_show (main_window);
+  }
+
+  DPRINTF ("Displaying first timestep\n",0);
   display_timestep (usermetacount, usermetanames, usermetadata);
 
+  DPRINTF ("Running main loop\n",0);
   gtk_main();
 
   DPRINTF ("Finalizing and exitting.\n",0);
